@@ -418,7 +418,7 @@ data_response.brmsterms <- function(x, data, check_response = TRUE,
 }
 
 # data specific for mixture models
-data_mixture <- function(bterms, prior = brmsprior()) {
+data_mixture <- function(bterms, data2, prior) {
   stopifnot(is.brmsterms(bterms))
   out <- list()
   if (is.mixfamily(bterms$family)) {
@@ -428,16 +428,8 @@ data_mixture <- function(bterms, prior = brmsprior()) {
       # estimate mixture probabilities directly
       take <- find_rows(prior, class = "theta", resp = bterms$resp)
       theta_prior <- prior$prior[take]
-      if (isTRUE(nzchar(theta_prior))) {
-        theta_prior <- eval_dirichlet(theta_prior)
-        if (length(theta_prior) != length(families)) {
-          stop2("Invalid dirichlet prior for the ", 
-                "mixture probabilities 'theta'.")
-        }
-        out$con_theta <- theta_prior
-      } else {
-        out$con_theta <- rep(1, length(families)) 
-      }
+      con_theta <- eval_dirichlet(theta_prior, length(families), data2)
+      out$con_theta <- as.array(con_theta)
       p <- usc(combine_prefix(bterms))
       names(out) <- paste0(names(out), p)
     }
@@ -446,7 +438,7 @@ data_mixture <- function(bterms, prior = brmsprior()) {
 }
 
 # data for the baseline functions of Cox models
-data_bhaz <- function(bterms, data, basis = NULL) {
+data_bhaz <- function(bterms, data, data2, prior, basis = NULL) {
   out <- list()
   if (!is_cox(bterms$family)) {
     return(out) 
@@ -457,6 +449,9 @@ data_bhaz <- function(bterms, data, basis = NULL) {
   out$Zbhaz <- bhaz_basis_matrix(y, args, basis = bs) 
   out$Zcbhaz <- bhaz_basis_matrix(y, args, integrate = TRUE, basis = bs)
   out$Kbhaz <- NCOL(out$Zbhaz)
+  sbhaz_prior <- subset2(prior, class = "sbhaz", resp = bterms$resp)
+  con_sbhaz <- eval_dirichlet(sbhaz_prior$prior, out$Kbhaz, data2)
+  out$con_sbhaz <- as.array(con_sbhaz)
   out
 }
 
